//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: SqlClient.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Xml;

using W3C.Soap;

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.Core.DsspHttp;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;

using store = Microsoft.Robotics.Services.DataBases.AdoNet.SqlStore;

namespace Microsoft.Robotics.Services.DataBases.AdoNet.SqlClient
{
    /// <summary>
    /// Implementation class for an SqlConnection service wrapper. 
    /// We explicitly request not to share the dispatcher as we may block our thread due to SQL interactions.
    /// </summary>
    [DisplayName("SQL Client for ADO.Net")]
    [Description("Provides access to an SQL Client using ADO.Net.")]
    [Contract(Contract.Identifier)]
    [ActivationSettings(ShareDispatcher = false)]
    public class SqlClientService : DsspServiceBase
    {
        /// <summary>
        /// Service instance state containing the SQL connection information. Information is persisted to file
        /// </summary>
        [InitialStatePartner(Optional = true, ServiceUri = ServicePaths.Store + @"/SqlClient.xml")]
        private SqlClientState _clientState = null;

        /// <summary>
        /// Store state contains DataSet
        /// </summary>
        private store.SqlStoreState _storeState = null;

        /// <summary>
        /// Main Port where we manage the overall connection string to the database
        /// </summary>
        [ServicePort("/sqlclient", AllowMultipleInstances = true)]
        private SqlClientOperations _pMain = new SqlClientOperations();

        [AlternateServicePort("/sqlstore", AllowMultipleInstances = true, AlternateContract = store.Contract.Identifier)]
        private store.SqlStoreOperations _pStore = new store.SqlStoreOperations();

        // We create a global SqlDataProviders which we use for all access to the database
        private SqlDataAdapter _sqlDataAdapter;

        // Database initialization sets up the SQL Client and SQL Store service state documents
        #region Database initialization
        /// <summary>
        /// Create a default database setup that talks to the Northwind database
        /// </summary>
        /// <returns>An initialized SqlClientState object</returns>
        private SqlClientState InitializeDefaultClientState()
        {
            SqlClientState cState = new SqlClientState();

            cState = new SqlClientState();

            // Initialize to default Northwind database
            cState.ConnectionString = @"server=(local)\SQLExpress;Integrated Security=SSPI;database=northwind";
            cState.CommandTimeOut = 30;

            // Use this as XML Namespace and outer XML element name respectively
            cState.DataNamespace = @"http://schemas.microsoft.com/robotics/2007/06/northwind.html";
            cState.DataSetName = @"Store";
            cState.DataTableName = @"Customers";

            cState.StoreXmlTransform = @"/mountpoint/samples/misc/databases/adonet/sqlclient/northwind.xslt";

            cState.AutoGenerateCommands = false;
            cState.MissingSchemaAction = MissingSchemaAction.AddWithKey;

            cState.SelectCommand = new SqlClientCommmand(@"SELECT * FROM Customers");

            cState.InsertCommand = new SqlClientCommmand(@"INSERT INTO Customers (CustomerID, CompanyName) Values(@CustomerID, @CompanyName)");
            cState.InsertCommand.ParameterList.Add(new SqlClientParameter("@CustomerID", SqlDbType.NChar, 5, "CustomerID"));
            cState.InsertCommand.ParameterList.Add(new SqlClientParameter("@CompanyName", SqlDbType.NChar, 15, "CompanyName"));

            cState.UpdateCommand = new SqlClientCommmand(@"UPDATE Customers Set CustomerID = @CustomerID, CompanyName = @CompanyName WHERE CustomerID = @OldCustomerID");
            cState.UpdateCommand.ParameterList.Add(new SqlClientParameter("@CustomerID", SqlDbType.NChar, 5, "CustomerID"));
            cState.UpdateCommand.ParameterList.Add(new SqlClientParameter("@CompanyName", SqlDbType.NChar, 15, "CompanyName"));
            cState.UpdateCommand.ParameterList.Add(new SqlClientParameter("@OldCustomerID", SqlDbType.NChar, 5, "CustomerID", DataRowVersion.Original));

            cState.DeleteCommand = new SqlClientCommmand(@"DELETE FROM Customers WHERE CustomerID = @CustomerID");
            cState.DeleteCommand.ParameterList.Add(new SqlClientParameter("@CustomerID", SqlDbType.NChar, 5, "CustomerID", DataRowVersion.Original));

            return cState;
        }

        private SqlCommand CreateCommand(SqlClientCommmand command, SqlConnection connection)
        {
            if (command != null)
            {
                SqlCommand sqlCmd = new SqlCommand(command.CommandString, connection);

                if (_clientState.CommandTimeOut > 0)
                {
                    sqlCmd.CommandTimeout = _clientState.CommandTimeOut;
                }

                if (command.ParameterList != null)
                {
                    SqlParameter parameter;
                    foreach (SqlClientParameter p in command.ParameterList)
                    {
                        parameter = sqlCmd.Parameters.Add(p.ParameterName, p.DataType, p.Size, p.SourceColumn);
                        parameter.SourceVersion = p.SourceVersion;
                    }
                }

                return sqlCmd;
            }
            return null;
        }

        private SqlDataAdapter InitializeStoreAdapter()
        {
            // Set up SqlDataAdapter object
            SqlDataAdapter sqlDataAdapter = new SqlDataAdapter();

            // Set MissingSchemaAction
            sqlDataAdapter.MissingSchemaAction = _clientState.MissingSchemaAction;

            // Set up SqlConnection object
            SqlConnection sqlConnection = new SqlConnection(_clientState.ConnectionString);

            // Set Select command
            sqlDataAdapter.SelectCommand = CreateCommand(_clientState.SelectCommand, sqlConnection);

            // Check whether we should autogenerate commands for UPDATE, INSERT, and DELETE
            if (_clientState.AutoGenerateCommands)
            {
                SqlCommandBuilder builder = new SqlCommandBuilder(sqlDataAdapter);
                builder.QuotePrefix = "[";
                builder.QuoteSuffix = "]";
            }
            else
            {
                // Set Insert command
                sqlDataAdapter.InsertCommand = CreateCommand(_clientState.InsertCommand, sqlConnection);

                // Set Update command
                sqlDataAdapter.UpdateCommand = CreateCommand(_clientState.UpdateCommand, sqlConnection);

                // Set Delete command
                sqlDataAdapter.DeleteCommand = CreateCommand(_clientState.DeleteCommand, sqlConnection);
            }
            return sqlDataAdapter;
        }

        /// <summary>
        /// Set up SQL Data Provider using information in SQL Client State
        /// </summary>
        /// <returns>An initialized SqlClientState object</returns>
        private store.SqlStoreState InitializeStoreState()
        {
            store.SqlStoreState sState = new store.SqlStoreState();

            // If we have a select command then fill the DataSet
            if (_clientState.SelectCommand != null)
            {
                // Create a data set and fill it in
                sState.Store = new DataSet(_clientState.DataSetName);
                if (!string.IsNullOrEmpty(_clientState.DataNamespace))
                {
                    sState.Store.Namespace = _clientState.DataNamespace;
                }
                _sqlDataAdapter.Fill(sState.Store, _clientState.DataTableName);
            }
            return sState;
        }
        #endregion

        // Service initialization handles service creation and start procedures
        #region Service Initialization
        /// <summary>
        /// Default Service Constructor
        /// </summary>
        public SqlClientService(DsspServiceCreationPort creationPort)
            :
                base(creationPort)
        {
        }

        /// <summary>
        /// Service Start
        /// </summary>
        protected override void Start()
        {
            // First check whether we had initial state or have to create it
            if (_clientState == null)
            {
                _clientState = InitializeDefaultClientState();

                // Persist state to disk.
                Activate(Arbiter.Choice(base.SaveState(_clientState),
                    delegate(DefaultReplaceResponseType success) { },
                    delegate(W3C.Soap.Fault fault)
                    {
                        LogError(null, "Unable to store state", fault);
                    }
                ));
            }

            try
            {
                // If connection string is valid then continue with setup.
                SqlConnectionStringBuilder SqlConString = new SqlConnectionStringBuilder(_clientState.ConnectionString);

                // Set up the SQL data adapter with our various commands for GET, UPDATE, INSERT, etc.
                _sqlDataAdapter = InitializeStoreAdapter();

                // Initialize store state based on SQL client state
                _storeState = InitializeStoreState();

                // Now continue with remainder of service initialization
                base.Start();
            }
            catch (Exception e)
            {
                LogError(e);
            }
        }
        #endregion

        // The SQL Client handlers operate on the state describing which data base we talk to and how
        #region SQL Client handlers
        #region HTTP
        /// <summary>
        /// HTTP Get Handler
        /// </summary>
        /// <param name="httpGet">The HTTP GET request</param>
        [ServiceHandler(ServiceHandlerBehavior.Concurrent, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreHttpGetHandler(HttpGet httpGet)
        {
            httpGet.ResponsePort.Post(
                new HttpResponseType(System.Net.HttpStatusCode.OK, _storeState, _clientState.StoreXmlTransform));
            yield break;
        }
        #endregion HTTP

        #region DSSP
        /// <summary>
        /// Get Handler
        /// </summary>
        /// <param name="get"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Concurrent)]
        public virtual IEnumerator<ITask> ClientGetHandler(Get get)
        {
            get.ResponsePort.Post(_clientState);
            yield break;
        }
        #endregion DSSP
        #endregion

        // The SQL Store handlers operate on the actual database by adding/updating/removing contents
        #region SQL Store handlers

        /// <summary>
        /// Store Get Handler
        /// </summary>
        /// <param name="get"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Concurrent, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreGetHandler(store.Get get)
        {
            get.ResponsePort.Post(_storeState);
            yield break;
        }

        /// <summary>
        /// Store Query Handler
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Concurrent, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreQueryHandler(store.Query query)
        {
            using (SqlConnection connection = new SqlConnection(_clientState.ConnectionString))
            {
                SqlDataAdapter adapter = new SqlDataAdapter();
                adapter.SelectCommand = new SqlCommand(query.Body.Query, connection);
                DataSet dataset = new DataSet(_clientState.DataSetName);
                adapter.Fill(dataset, _clientState.DataTableName);
                store.SqlStoreState queryResponse = new store.SqlStoreState();
                queryResponse.Store = dataset;
                query.ResponsePort.Post(queryResponse);
            }
            yield break;
        }

        /// <summary>
        /// Shared function for performing INSERT, UPDATE, UPSERT, and DELETE on the dataset
        /// </summary>
        /// <param name="dataSet">Request DataSet contaning modifications</param>
        /// <param name="modification">Type of modification to apply</param>
        /// <param name="fault">If the modifications fail fill in the fault with details</param>
        /// <returns>True if modifications were successfull; otherwise false.</returns>
        private bool StoreApplyChange(DataSet dataSet, DataRowState modification, ref Fault fault)
        {
            // Check that we have modifications to apply
            if (dataSet == null || !dataSet.HasChanges(modification))
            {
                fault =
                    Fault.FromCodeSubcodeReason(
                        FaultCodes.Sender,
                        DsspFaultCodes.OperationFailed,
                        "State change request contained no data rows to update.");
                return false;
            }

            // Get the part of the dataset that have the right modification
            DataSet modSet = dataSet.GetChanges(modification);

            // Check the update DataSet for errors
            if (modSet.HasErrors)
            {
                // Create object to hold errors
                store.SqlStoreError errors = new store.SqlStoreError();

                // Collect all errors in our modifications
                foreach (DataTable table in modSet.Tables)
                {
                    // If the table has errors, then print them.
                    if (table.HasErrors)
                    {
                        foreach (DataRow row in table.Rows)
                        {
                            if (row.HasErrors)
                            {
                                errors.RowErrorList.Add(row.RowError);
                            }
                        }
                    }
                }

                // Generate SOAP fault containing the modification errors
                fault =
                    Fault.FromCodeSubcodeReason(
                        FaultCodes.Sender,
                        DsspFaultCodes.OperationFailed,
                        "State change request had errors in contained no data rows to update.");

                // Add errors in detail of SOAP fault
                fault.Detail = new Detail();
                fault.Detail.Any = new object[1];
                fault.Detail.Any[0] = errors;
                return false;
            }

            try
            {
                // Update database
                _sqlDataAdapter.Update(modSet, _clientState.DataTableName);

                // Update our store dataset to represent new values (and any other changes in the database)
                _sqlDataAdapter.Fill(_storeState.Store, _clientState.DataTableName);
            }
            catch (Exception e)
            {
                fault = Fault.FromException(e);
                return false;
            }
            return true;
        }

        /// <summary>
        /// Store Insert Handler
        /// </summary>
        /// <param name="update"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreInsertHandler(store.Insert insert)
        {
            Fault fault = null;
            if (StoreApplyChange(insert.Body.Store, DataRowState.Added, ref fault))
            {
                // Post successful response
                insert.ResponsePort.Post(DefaultInsertResponseType.Instance);
            }
            else
            {
                // Post SOAP fault
                insert.ResponsePort.Post(fault);
            }
            yield break;
        }

        /// <summary>
        /// Store Update Handler
        /// </summary>
        /// <param name="update"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreUpdateHandler(store.Update update)
        {
            Fault fault = null;
            if (StoreApplyChange(update.Body.Store, DataRowState.Modified, ref fault))
            {
                // Post successful response
                update.ResponsePort.Post(DefaultUpdateResponseType.Instance);
            }
            else
            {
                // Post SOAP fault
                update.ResponsePort.Post(fault);
            }
            yield break;
        }

        /// <summary>
        /// Store Upsert Handler
        /// </summary>
        /// <param name="upsert"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreUpsertHandler(store.Upsert upsert)
        {
            Fault fault = null;
            if (StoreApplyChange(upsert.Body.Store, DataRowState.Added | DataRowState.Modified, ref fault))
            {
                // Post successful response
                upsert.ResponsePort.Post(DefaultUpsertResponseType.Instance);
            }
            else
            {
                // Post SOAP fault
                upsert.ResponsePort.Post(fault);
            }
            yield break;
        }

        /// <summary>
        /// Store Delete Handler
        /// </summary>
        /// <param name="upsert"></param>
        /// <returns></returns>
        [ServiceHandler(ServiceHandlerBehavior.Exclusive, PortFieldName = "_pStore")]
        public virtual IEnumerator<ITask> StoreDeleteHandler(store.Delete delete)
        {
            Fault fault = null;
            if (StoreApplyChange(delete.Body.Store, DataRowState.Deleted, ref fault))
            {
                // Post successful response
                delete.ResponsePort.Post(DefaultDeleteResponseType.Instance);
            }
            else
            {
                // Post SOAP fault
                delete.ResponsePort.Post(fault);
            }
            yield break;
        }

        #endregion
    }
}
