//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: SqlClientTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;

using W3C.Soap;

namespace Microsoft.Robotics.Services.DataBases.AdoNet.SqlClient
{
    /// <summary>
    /// SqlClient Contract class
    /// 
    /// Generic SQL client service
    /// The SQL client service enables generic access to an SQL store through the SQL store service.
 
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// The Dss Service contract
        /// </summary>
        public const String Identifier = "http://schemas.microsoft.com/robotics/2007/06/sqlclient.html";
    }

    /// <summary>
    /// The SqlClient State
    /// </summary>
    [DataContract]
    [Description ("Identifies the configuration of the SQL client.")]
    public class SqlClientState
    {
        /// <summary>
        /// The SQL connection string used to connect to the database
        /// </summary>
        [DataMember]
        [Description("Specifies the connection string used to connect to the database.")]
        public string ConnectionString;

        /// <summary>
        /// The time in seconds to wait for the command to execute. The default is 30 seconds. 
        /// </summary>
        [DataMember]
        [Description("Specifies the time in seconds to wait for the command to execute.\n(Default is 30 seconds)")]
        public int CommandTimeOut;

        /// <summary>
        /// The XML Namespace to use for generated XML.
        /// </summary>
        [DataMember]
        [Description("Specifies XML namespace to use for the generated XML.")]
        public string DataNamespace;

        /// <summary>
        /// A name for the DataSet is required to ensure that the XML representation of the
        /// DataSet always has a name for the document element, which is the highest level
        /// element in a schema definition. If not set then the default value is used.
        /// </summary>
        [DataMember]
        [Description("Specifies a name for the DataSet.")]
        public string DataSetName;

        /// <summary>
        /// Name to use for DataTable entries when generating a DataSet. If not set then the 
        /// default value is used.
        /// </summary>
        [DataMember]
        [Description("Specifies a name to use for DataTable entries when generating a DataSet.")]
        public string DataTableName;

        /// <summary>
        /// URI of XSLT to apply to a GET response from the store.
        /// </summary>
        [DataMember]
        [Description("Specifies the URI of an XSLT to format the date returned from the store.")]
        public string StoreXmlTransform;

        /// <summary>
        /// Determines the action to take when existing DataSet schema does not match data in database. 
        /// </summary>
        [DataMember]
        [Description("Specifies an action to take when the existing DataSet schema does not match data in the database.")]
        public MissingSchemaAction MissingSchemaAction;

        /// <summary>
        /// SQL command for retrieving all data from database.
        /// </summary>
        [DataMember]
        [Description("Specifies an SQL commmand for retrieving all data from the database.")]
        public SqlClientCommmand SelectCommand;

        /// <summary>
        /// Indicate whether the commands for UPDATE, INSERT, DELETE should be autogenerated using
        /// SqlCommandBuilder based on the contents of the SelectCommand. If not then UpdateCommand,
        /// InsertCommand, and DeleteCommand are used instead.
        /// </summary>
        [DataMember]
        [Description("Specifies whether to autogenerate UPDATE, INSERT, and DELETE commands.")]
        public bool AutoGenerateCommands;

        /// <summary>
        /// SQL command for updating data in database.
        /// </summary>
        [DataMember]
        [Description("Specifies the SQL command for updating data in the database.")]
        public SqlClientCommmand UpdateCommand;

        /// <summary>
        /// SQL command for inserting data into database.
        /// </summary>
        [DataMember]
        [Description("Specifies the SQL command for inserting data into the database.")]
        public SqlClientCommmand InsertCommand;

        /// <summary>
        /// SQL command for deleting data from database.
        /// </summary>
        [DataMember]
        [Description("Specifies the SQL command for deleting data from the database.")]
        public SqlClientCommmand DeleteCommand;
    }

    [DataContract]
    public class SqlClientCommmand
    {
        [DataMember]
        [Description("Specifies the string used to open an SQL Server database.")]
        public string CommandString;

        [DataMember]
        [Description("Identifies a set of parameters to apply to open an SQL Server database.")]
        public List<SqlClientParameter> ParameterList = new List<SqlClientParameter>();

        public SqlClientCommmand()
        {
        }

        public SqlClientCommmand(string commandString)
        {
            CommandString = commandString;
        }
    }

    [DataContract]
    public class SqlClientParameter
    {
        [DataMember]
        [Description("Specifies the name of the parameter.")]
        public string ParameterName;

        [DataMember]
        [Description("Specifies the type of the parameter.")]
        public SqlDbType DataType;

        [DataMember]
        [Description("Specifies the maximum size, in bytes, of the data within the column.")]
        public int Size;

        [DataMember]
        [Description("Specifies the name of the source column mapped to the DataSet.")]
        public string SourceColumn;

        [DataMember]
        [Description("Specifies the version of the DataRow.")]
        public DataRowVersion SourceVersion;

        public SqlClientParameter()
        {
        }

        public SqlClientParameter(string parameterName, SqlDbType dataType, int size, string sourceColumn)
            :
            this(parameterName, dataType, size, sourceColumn, DataRowVersion.Current)
        {
        }

        public SqlClientParameter(string parameterName, SqlDbType dataType, int size, string sourceColumn, DataRowVersion sourceVersion)
        {
            ParameterName = parameterName;
            DataType = dataType;
            Size = size;
            SourceColumn = sourceColumn;
            SourceVersion = sourceVersion;
        }
    }

    /// <summary>
    /// SqlClient Main Operations Port
    /// </summary>
    [ServicePort]
    public class SqlClientOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get>
    {
    }

    /// <summary>
    /// SqlConnection Get Operation
    /// </summary>
    [Description("Gets the current state of the SQL connection.")]
    public class Get : Get<GetRequestType, PortSet<SqlClientState, Fault>>
    {
    }
}
