//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: DirectionDialogTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using W3C.Soap;

using directiondialog = Microsoft.Robotics.Services.Sample.DirectionDialog;


namespace Microsoft.Robotics.Services.Sample.DirectionDialog
{
    
    /// <summary>
    /// DirectionDialog Contract
    /// </summary>
    public static class Contract
    {
        /// The Unique Contract Identifier for the DirectionDialog service
        public const String Identifier = "http://schemas.microsoft.com/robotics/2006/08/directiondialog.html";
    }

    [DataContract]
    public class DirectionDialogState
    {
        private List<Button> _buttons = new List<Button>();

        [DataMember(IsRequired = true)]
        public List<Button> Buttons
        {
            get { return _buttons ; }
            set { _buttons  = value; }
        }
    }

    /// <summary>
    /// Button Direction
    /// </summary>
    [DataContract]
    public enum ButtonDirection
    {
        /// <summary>
        /// Stop
        /// </summary>
        Stop = 0,
        /// <summary>
        /// Forwards
        /// </summary>
        Forwards,
        /// <summary>
        /// Backwards
        /// </summary>
        Backwards,
        /// <summary>
        /// Left
        /// </summary>
        Left,
        /// <summary>
        /// Right
        /// </summary>
        Right,
    }

    [DataContract]
    public class Button
    {
        private bool _pressed;
        private ButtonDirection _direction;
        
        public Button()
        {
        }

        public Button(string name)
        {
            this.Name = name;
        }

        /// <summary>
        /// The button direction
        /// </summary>
        /// <param name="direction"></param>
        public Button(ButtonDirection direction)
        {
            _direction = direction;
        }

        /// <summary>
        /// The Button Name
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _direction.ToString(); }
            set 
            { 
                try
                {
                    _direction = (ButtonDirection)Enum.Parse(typeof(ButtonDirection), value);
                }
                catch
                {
                    _direction = ButtonDirection.Stop;
                }
            }
        }

        /// <summary>
        /// The Button Direction
        /// </summary>
        [DataMember, DataMemberConstructor]
        public ButtonDirection Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }

        [DataMember]
        public bool Pressed
        {
            get { return _pressed; }
            set { _pressed = value; }
        }
    }

    [DataContract]
    public class ButtonPressRequest
    {
        public ButtonPressRequest()
        {
        }

        public ButtonPressRequest(string name)
        {
            this.Name = name;
        }

        public ButtonPressRequest(ButtonDirection direction)
        {
            this._direction = direction;
        }

        private ButtonDirection _direction;

        /// <summary>
        /// The Button Name
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _direction.ToString(); }
            set
            {
                try
                {
                    _direction = (ButtonDirection)Enum.Parse(typeof(ButtonDirection), value);
                }
                catch
                {
                    _direction = ButtonDirection.Stop;
                }
            }
        }

        /// <summary>
        /// The Button Direction
        /// </summary>
        [DataMember, DataMemberConstructor]
        public ButtonDirection Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }

    }

    [DataContract]
    public class ButtonReleaseRequest
    {
        public ButtonReleaseRequest()
        {
        }

        public ButtonReleaseRequest(string name)
        {
            this.Name = name;
        }

        private ButtonDirection _direction;

        /// <summary>
        /// The Button Name
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _direction.ToString(); }
            set
            {
                try
                {
                    _direction = (ButtonDirection)Enum.Parse(typeof(ButtonDirection), value);
                }
                catch
                {
                    _direction = ButtonDirection.Stop;
                }
            }
        }

        /// <summary>
        /// The Button Direction
        /// </summary>
        [DataMember, DataMemberConstructor]
        public ButtonDirection Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }
    }

    [ServicePort]
    public class DirectionDialogOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Replace, ButtonPress, ButtonRelease, Subscribe>
    {
    }

    [DisplayName("Get")]
    [Description("Gets the current state of the dialog as a list of the buttons on the dialog and their current pressed states.")]
    public class Get : Get<GetRequestType, PortSet<DirectionDialogState, Fault>>
    {
    }

    [DisplayName("DialogStateChange")]
    [Description("Indicates when the dialog state changes.")]
    public class Replace : Replace<DirectionDialogState, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }

    [DisplayName("ButtonPress")]
    [Description("Indicates when a button in the dialog is pressed.")]
    public class ButtonPress : Update<ButtonPressRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public ButtonPress()
        {
        }

        public ButtonPress(ButtonPressRequest body)
            : base(body)
        { 
        }
    }

    [DisplayName("ButtonRelease")]
    [Description("Indicates when a button in the dialog is released.")]
    public class ButtonRelease : Update<ButtonReleaseRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public ButtonRelease()
        {
        }

        public ButtonRelease(ButtonReleaseRequest body)
            : base(body)
        {
        }
    }

    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }
}
