//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: AtomTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Xml;
using System.Xml.Serialization;
using System.ComponentModel;

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.Core.DsspHttp;
using Microsoft.Dss.ServiceModel.Dssp;

using W3C.Soap;

namespace Microsoft.Robotics.Services.Syndication.Atom
{
    #region ServiceContract
    /// <summary>
    /// Atom Contract identifying this service
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// The Unique Contract Identifier for the Atom service
        /// </summary>
        public const String Identifier = "http://schemas.microsoft.com/robotics/2007/01/atom.html";
    }
    #endregion

    #region ServiceState
    /// <summary>
    /// An Atom Feed Document is a representation of an Atom feed, including
    /// metadata about the feed, and some or all of the entries associated
    /// with it.  Its root is the atom:feed element.
    /// </summary>
    [DataContract(Name = "feed", Namespace = "http://www.w3.org/2005/Atom")]
    public class AtomFeed
    {
        /// <summary>
        /// The "atom:id" element conveys a permanent, universally unique identifier 
        /// for an entry or feed. Its content MUST be an IRI, as defined by [RFC3987].
        /// Note that the definition of "IRI" excludes relative references.  Though the
        /// IRI might use a dereferencable scheme, Atom Processors MUST NOT assume
        /// it can be dereferenced.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string id;

        /// <summary>
        /// The "atom:title" element is a Text construct that conveys a human-
        /// readable title for an entry or feed.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string title;

        /// <summary>
        /// The "atom:updated" element is a Date construct indicating the most
        /// recent instant in time when an entry or feed was modified in a way
        /// the publisher considers significant.  Therefore, not all
        /// modifications necessarily result in a changed atom:updated value.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string updated;

        /// <summary>
        /// The author element is a Person construct that indicates the
        /// author of the entry or feed.
        /// If an atom:entry element does not contain atom:author elements, then
        /// the atom:author elements of the contained atom:source element are
        /// considered to apply.  In an Atom Feed Document, the atom:author
        /// elements of the containing atom:feed element are considered to apply
        /// to the entry if there are no atom:author elements in the locations
        /// described above.
        /// </summary>
        [DataMember]
        public AtomPerson author;

        /// <summary>
        /// Identifies a related Web page. The type of relation is defined by the rel attribute.
        /// A feed is limited to one alternate per type and hreflang.
        /// A feed should contain a link back to the feed itself.
        /// </summary>
        [DataMember(Name = "link", XmlInlineList = true)]
        public List<AtomLink> LinkList;

        /// <summary>
        /// List of entries for this feed
        /// </summary>
        [DataMember(Name = "entry", XmlInlineList = true)]
        public List<AtomEntry> EntryList;

        /// <summary>
        /// Default constructor
        /// </summary>
        public AtomFeed()
        { }

        /// <summary>
        /// Constructor with initialization
        /// </summary>
        /// <param name="feedTitle">Title of the feed</param>
        public AtomFeed(string feedTitle)
        {
            id = "urn:uuid:" + Guid.NewGuid().ToString("D");
            title = feedTitle;
            updated = DateTime.UtcNow.ToString("o");
            LinkList = new List<AtomLink>();
            EntryList = new List<AtomEntry>();
        }

        /// <summary>
        /// Constructor with initialization
        /// </summary>
        /// <param name="source">Feed to initialize this feed with</param>
        public AtomFeed(AtomFeed source)
        {
            this.id = source.id;
            this.title = source.title;
            this.updated = source.updated;
            this.author = source.author;
            this.LinkList = source.LinkList;
            this.EntryList = source.EntryList;
        }
    }

    /// <summary>
    /// The "atom:entry" element represents an individual entry, acting as a
    /// container for metadata and data associated with the entry.  This
    /// element can appear as a child of the atom:feed element, or it can
    /// appear as the document (i.e., top-level) element of a stand-alone
    /// Atom Entry Document
    /// </summary>
    [DataContract(Name = "entry", Namespace = "http://www.w3.org/2005/Atom")]
    public class AtomEntry
    {
        /// <summary>
        /// The "atom:id" element conveys a permanent, universally unique identifier 
        /// for an entry or feed. Its content MUST be an IRI, as defined by [RFC3987].
        /// Note that the definition of "IRI" excludes relative references.  Though the
        /// IRI might use a dereferencable scheme, Atom Processors MUST NOT assume
        /// it can be dereferenced.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string id;

        /// <summary>
        /// The "atom:title" element is a Text construct that conveys a human-
        /// readable title for an entry or feed.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string title;

        /// <summary>
        /// The "atom:updated" element is a Date construct indicating the most
        /// recent instant in time when an entry or feed was modified in a way
        /// the publisher considers significant.  Therefore, not all
        /// modifications necessarily result in a changed atom:updated value.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string updated;

        /// <summary>
        /// The author element is a Person construct that indicates the
        /// author of the entry or feed.
        /// If an atom:entry element does not contain atom:author elements, then
        /// the atom:author elements of the contained atom:source element are
        /// considered to apply.  In an Atom Feed Document, the atom:author
        /// elements of the containing atom:feed element are considered to apply
        /// to the entry if there are no atom:author elements in the locations
        /// described above.
        /// </summary>
        [DataMember]
        public AtomPerson author;

        /// Identifies a related Web page. The type of relation is defined by the rel attribute.
        /// A feed is limited to one alternate per type and hreflang.
        /// A feed should contain a link back to the feed itself.
        [DataMember(Name = "link", XmlInlineList = true)]
        public List<AtomLink> LinkList;

        /// <summary>
        /// The content element either contains or links to the content of
        /// the entry.  The content of atom:content is Language-Sensitive.
        /// </summary>
        [DataMember]
        public AtomContent content;

        /// <summary>
        /// Default constructor
        /// </summary>
        public AtomEntry()
        { }

        /// <summary>
        /// Constructor with initialization
        /// </summary>
        /// <param name="entryTitle">Title of the entry</param>
        /// <param name="entryContent">Content of the entry</param>
        public AtomEntry(string entryTitle, AtomContent entryContent)
        {
            id = "urn:uuid:" + Guid.NewGuid().ToString("D");
            updated = DateTime.UtcNow.ToString("o");
            title = entryTitle;
            LinkList = new List<AtomLink>();
            content = entryContent;
        }

        /// <summary>
        /// Compares two AtomEntry objects by doing an ordinal string comparison on
        /// their respective id fields.
        /// </summary>
        /// <param name="entry"></param>
        /// <returns></returns>
        public bool Equals(AtomEntry entry)
        {
            return (string.CompareOrdinal(entry.id, this.id) == 0);
        }
    }

    /// <summary>
    /// A Person construct is an element that describes a person,
    /// corporation, or similar entity.
    /// </summary>
    [DataContract]
    public class AtomPerson
    {
        /// <summary>
        /// The name element's content conveys a human-readable name for
        /// the person.  The content of atom:name is Language-Sensitive.  Person
        /// constructs MUST contain exactly one name element.
        /// </summary>
        [DataMember(IsRequired = true)]
        public string name;

        /// <summary>
        /// The email element's content conveys an e-mail address
        /// associated with the person.  Person constructs MAY contain an
        /// email element, but MUST NOT contain more than one.  Its content
        /// MUST conform to the "addr-spec" production in [RFC2822].
        /// </summary>
        [DataMember]
        public string email;

        /// <summary>
        /// The uri element's content conveys an IRI associated with the
        /// person.  Person constructs MAY contain an atom:uri element, but MUST
        /// NOT contain more than one.  The content of uri in a Person
        /// construct MUST be an IRI reference.
        /// </summary>
        [DataMember]
        public string uri;
    }

    /// <summary>
    /// Identifies a related Web page. The type of relation is defined by the rel attribute.
    /// A feed is limited to one alternate per type and hreflang.
    /// A feed should contain a link back to the feed itself.
    /// </summary>
    [DataContract]
    public class AtomLink
    {
        /// <summary>
        /// The "href" attribute contains the link's IRI. atom:link elements MUST
        /// have an href attribute, whose value MUST be a IRI reference
        /// [RFC3987]
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string href;

        /// <summary>
        ///  link elements MAY have a "rel" attribute that indicates the link
        ///  relation type.  If the "rel" attribute is not present, the link
        ///  element MUST be interpreted as if the link relation type is
        ///  "alternate".
        ///  The value of "rel" MUST be a string that is non-empty and matches
        ///  either the "isegment-nz-nc" in [RFC4287] or the "IRI" production in [RFC3987].
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string rel;

        /// <summary>
        /// On the link element, the "type" attribute's value is an advisory
        /// media type: it is a hint about the type of the representation that is
        /// expected to be returned when the value of the href attribute is
        /// dereferenced.  Note that the type attribute does not override the
        /// actual media type returned with the representation.  Link elements
        /// MAY have a type attribute, whose value MUST conform to the syntax of
        /// a MIME media type 
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string type;

        /// <summary>
        /// The "hreflang" attribute's content describes the language of the
        /// resource pointed to by the href attribute.  When used together with
        /// the rel="alternate", it implies a translated version of the entry.
        /// Link elements MAY have an hreflang attribute, whose value MUST be a
        /// language tag [RFC3066]
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string hreflang;

        /// <summary>
        /// The "title" attribute conveys human-readable information about the
        /// link.  The content of the "title" attribute is Language-Sensitive.
        /// Entities such as "&amp;" and "&lt;" represent their corresponding
        /// characters ("&amp;" and "&amp;", respectively), not markup.  Link elements
        /// MAY have a title attribute
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string title;

        /// <summary>
        /// The "length" attribute indicates an advisory length of the linked
        /// content in octets; it is a hint about the content length of the
        /// representation returned when the IRI in the href attribute is mapped
        /// to a URI and dereferenced.  Note that the length attribute does not
        /// override the actual content length of the representation as reported
        /// by the underlying protocol.  Link elements MAY have a length
        /// attribute
        /// </summary>
        [DataMember(XmlAttribute = true, XmlOmitDefaultValue = true)]
        public int length;

        ///// <summary>
        ///// Indicates whether the length should be serialized or not
        ///// </summary>
        //[DataMember()]
        //[XmlIgnore]
        //public bool lengthSpecified;

        /// <summary>
        /// Default constructor
        /// </summary>
        public AtomLink() { }

        /// <summary>
        /// <param name="linkHref">The "href" attribute contains the link's IRI. atom:link elements MUST
        /// have an href attribute, whose value MUST be a IRI reference
        /// [RFC3987]</param>
        /// <param name="linkRel">On the link element, the "type" attribute's value is an advisory
        /// media type: it is a hint about the type of the representation that is
        /// expected to be returned when the value of the href attribute is
        /// dereferenced.  Note that the type attribute does not override the
        /// actual media type returned with the representation.  Link elements
        /// MAY have a type attribute, whose value MUST conform to the syntax of
        /// a MIME media type</param>
        /// </summary>
        public AtomLink(string linkHref, string linkRel)
        {
            href = linkHref;
            rel = linkRel;
        }
    }

    /// <summary>
    /// The content element either contains or links to the content of
    /// the entry.  The content of atom:content is Language-Sensitive
    /// </summary>
    [DataContract]
    public class AtomContent
    {
        /// <summary>
        /// The "atom:content" element either contains or links to the content of
        /// the entry.  The content of atom:content is Language-Sensitive
        /// </summary>
        [DataMember(XmlText = true)]
        public string content;

        /// <summary>
        /// On the atom:content element, the value of the "type" attribute MAY be
        /// one of "text", "html", or "xhtml".  Failing that, it MUST conform to
        /// the syntax of a MIME media type, but MUST NOT be a composite type
        /// (see Section 4.2.6 of [MIMEREG]).  If neither the type attribute nor
        /// the src attribute is provided, Atom Processors MUST behave as though
        /// the type attribute were present with a value of "text".
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string type;

        /// <summary>
        /// atom:content MAY have a "src" attribute, whose value MUST be an IRI
        /// reference [RFC3987].  If the "src" attribute is present, atom:content
        /// MUST be empty.  Atom Processors MAY use the IRI to retrieve the
        /// content and MAY choose to ignore remote content or to present it in a
        /// different manner than local content.
        /// 
        /// If the "src" attribute is present, the "type" attribute SHOULD be
        /// provided and MUST be a MIME media type [MIMEREG], rather than "text",
        /// "html", or "xhtml".
        /// </summary>
        [DataMember(XmlAttribute = true)]
        public string src;

        /// <summary>
        /// Default constructor
        /// </summary>
        public AtomContent()
        {
        }

        /// <summary>
        /// The content element either contains or links to the content of
        /// the entry.  The content of atom:content is Language-Sensitive.
        /// </summary>
        /// <param name="entryContent">String representing the inline content</param>
        /// <param name="entryType">Type identifier of content.</param>
        public AtomContent(string entryContent, string entryType)
        {
            content = entryContent;
            type = entryType;
        }
    }

    /// <summary>
    /// Text constructs MAY have a "type" attribute.  When present, the value
    /// MUST be one of "text", "html", or "xhtml".  If the "type" attribute
    /// is not provided, Atom Processors MUST behave as though it were
    /// present with a value of "text".  Unlike the atom:content element
    /// defined in Section 4.1.3 [RFC4287], MIME media types [MIMEREG] MUST NOT be used
    /// as values for the "type" attribute on Text constructs
    /// </summary>
    public static class AtomType
    {
        /// <summary>
        /// If the value is "text", the content of the Text construct MUST NOT
        /// contain child elements.  Such text is intended to be presented to
        /// humans in a readable fashion.  Thus, Atom Processors MAY collapse
        /// white space (including line breaks) and display the text using
        /// typographic techniques such as justification and proportional 
        /// </summary>
        public const string Text = "text";

        /// <summary>
        /// If the value of "type" is "html", the content of the Text construct
        /// MUST NOT contain child elements and SHOULD be suitable for handling
        /// as HTML [HTML].  Any markup within MUST be escaped; for example,
        /// "&lt;br>" as "&amp;lt;br>".  HTML markup within SHOULD be such that it could
        /// validly appear directly within an HTML &lt;DIV> element, after
        /// unescaping.  Atom Processors that display such content MAY use that
        /// markup to aid in its display.
        /// </summary>
        public const string Html = "html";

        /// <summary>
        /// If the value of "type" is "xhtml", the content of the Text construct
        /// MUST be a single XHTML div element [XHTML] and SHOULD be suitable for
        /// handling as XHTML.  The XHTML div element itself MUST NOT be
        /// considered part of the content.  Atom Processors that display the
        /// content MAY use the markup to aid in displaying it.  The escaped
        /// versions of characters such as "&amp;" and ">" represent those
        /// characters, not markup.
        /// </summary>
        public const string XHtml = "xhtml";

        /// <summary>
        /// Indicates whether string of type "text"
        /// </summary>
        /// <param name="type">Type name to evaluate</param>
        /// <returns>true if type name is text</returns>
        public static bool IsText(string type)
        {
            return (String.Compare(AtomType.Text, type, StringComparison.InvariantCultureIgnoreCase) == 0);
        }

        /// <summary>
        /// Indicates whether string of type "html"
        /// </summary>
        /// <param name="type">Type name to evaluate</param>
        /// <returns>true if type name is HTML</returns>
        public static bool IsHtml(string type)
        {
            return (String.Compare(AtomType.Html, type, StringComparison.InvariantCultureIgnoreCase) == 0);
        }

        /// <summary>
        /// Indicates whether string of type "XHtml"
        /// </summary>
        /// <param name="type">Type name to evaluate</param>
        /// <returns>true if type name is XHtml</returns>
        public static bool IsXHtml(string type)
        {
            return (String.Compare(AtomType.XHtml, type, StringComparison.InvariantCultureIgnoreCase) == 0);
        }

        /// <summary>
        /// Canonicalizes an Atom entry type name to match one of the predefined types
        /// or as a lowercase media type.
        /// </summary>
        /// <param name="type">Type value to canonicalize</param>
        /// <returns>Canonicalized representation</returns>
        public static string Canonicalize(string type)
        {
            if (String.IsNullOrEmpty(type))
            {
                return "";
            }
            if (String.Compare(AtomType.Text, type, StringComparison.InvariantCultureIgnoreCase) == 0)
            {
                return AtomType.Text;
            }
            if (String.Compare(AtomType.Html, type, StringComparison.InvariantCultureIgnoreCase) == 0)
            {
                return AtomType.Html;
            }
            if (String.Compare(AtomType.XHtml, type, StringComparison.InvariantCultureIgnoreCase) == 0)
            {
                return AtomType.XHtml;
            }
            return type.Trim().ToLowerInvariant();
        }
    }

    /// <summary>
    /// link elements MAY have a "rel" attribute that indicates the link
    /// relation type.  If the "rel" attribute is not present, the link
    /// element MUST be interpreted as if the link relation type is
    /// "alternate". RFC 4287 defines five initial values for the
    /// Registry of Link Relations as listed here.
    /// </summary>
    public static class AtomLinkRelation
    {
        /// <summary>
        /// The value "alternate" signifies that the IRI in the value of the
        /// href attribute identifies an alternate version of the resource
        /// described by the containing element
        /// </summary>
        public const string Alternate = "alternate";

        /// <summary>
        /// The value "related" signifies that the IRI in the value of the
        /// href attribute identifies a resource related to the resource
        /// described by the containing element.  For example, the feed for a
        /// site that discusses the performance of the search engine at
        /// "http://search.example.com" might contain, as a child of
        /// atom:feed:
        /// &lt;link rel="related" href="http://search.example.com/"/&gt;
        /// An identical link might appear as a child of any atom:entry whose
        /// content contains a discussion of that same search engine.
        /// </summary>
        public const string Related = "related";

        /// <summary>
        /// The value "self" signifies that the IRI in the value of the href
        /// attribute identifies a resource equivalent to the containing
        /// element.
        /// </summary>
        public const string Self = "self";

        /// <summary>
        /// The value "enclosure" signifies that the IRI in the value of the
        /// href attribute identifies a related resource that is potentially
        /// large in size and might require special handling.  For atom:link
        /// elements with rel="enclosure", the length attribute SHOULD be
        /// provided.
        /// </summary>
        public const string Enclusure = "enclosure";

        /// <summary>
        /// The value "via" signifies that the IRI in the value of the href
        /// attribute identifies a resource that is the source of the
        /// information provided in the containing element.
        /// </summary>
        public const string Via = "via";
    }
    #endregion

    #region ServiceOperations
    /// <summary>
    /// Atom Operations
    /// </summary>
    public class AtomOperations :
        PortSet<DsspDefaultLookup, DsspDefaultDrop, Get, Replace, Subscribe, InsertEntry, UpdateFeed, UpdateEntry, DeleteEntry, HttpGet, HttpQuery, HttpPost>
    {
    }

    /// <summary>
    /// Get the feed provided by this service instance
    /// </summary>
    [Description ("Get the feed provided by the service.")]
    public class Get : Get<GetRequestType, PortSet<AtomFeed, Fault>>
    {
    }

    /// <summary>
    /// Replace the feed provided by this service instance
    /// </summary>
    [Description("Changes (or indicates a change to) the feed provided by this service.")]
    public class Replace : Replace<AtomFeed, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }

    /// <summary>
    /// Subscribe to changes to the feed
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    /// <summary>
    /// InsertEntry an entry into the feed
    /// </summary>
    [Description("Inserts an entry into the feed.")]
    public class InsertEntry : Insert<AtomEntry, PortSet<DefaultInsertResponseType, Fault>>
    {
    }

    /// <summary>
    /// Update the feed information
    /// </summary>
    [Description("Updates the feed information.")]
    public class UpdateFeed : Update<AtomFeed, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Update entry information
    /// </summary>
    [Description("Inserts an entry.")]
    public class UpdateEntry : Update<AtomEntry, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Delete entry information
    /// </summary>
    [Description("Deletes an entry.")]
    public class DeleteEntry : Delete<AtomEntry, PortSet<DefaultDeleteResponseType, Fault>>
    {
    }
    #endregion
}
