//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: MathTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Ccr.Core;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using W3C.Soap;

using math = System.Math;

namespace Microsoft.Robotics.Services.Sample.Math
{
    /// Math Contract
    public static class Contract
    {
        /// The Unique Contract Identifier for the Timer service
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/08/math.html";
    }

    [DataContract]
    public class Response
    {
        public Response()
        {
        }

        public Response(double value)
        {
            Result = value;
        }

        [DataMember]
        [Description("Indicates the result of the math operation.")]
        public double Result;
    }
    
    [DataContract]
    public class MathConstants
    {
        [DataMember]
        [Description("Specifies the value of pi.")]
        public double PI = math.PI;
        [DataMember]
        [Description("Specifies the value of E.")]
        public double E = math.E;
    }

    [DataContract]
    public class AcosRequest
    {
        [DataMember]
        [Description("Specifies a number representing a cosine, where D is a value between -1 and 1.")]
        public double D;
    }

    [DataContract]
    public class AsinRequest
    {
        [DataMember]
        [Description("Specifies a number representing a sine, where D is a value between -1 and 1.")]
        public double D;
    }

    [DataContract]
    public class AtanRequest
    {
        [DataMember]
        [Description("Specifies a number representing a tangent.")]
        public double D;
    }

    [DataContract]
    public class Atan2Request
    {
        [DataMember]
        [Description("Specifies the y coordinate of a point.")]
        public double Y;
        [DataMember]
        [Description("Specifies the x coordinate of a point.")]
        public double X;
    }

    [DataContract]
    public class CosRequest
    {
        [DataMember]
        [Description("Specifies an angle, measured in radians.")]
        public double Angle;
    }

    [DataContract]
    public class ExpRequest
    {
        [DataMember]
        [Description("Specifies a number specifying a power.")]
        public double Exponent;
    }

    [DataContract]
    public class LogRequest
    {
        [DataMember]
        [Description("Specifies a number whose logarithm is to be found.")]
        public double A;
        [DataMember]
        [Description("Specifies the base of the logarithm.")]
        public double Base;
    }

    [DataContract]
    public class PowRequest
    {
        [DataMember]
        [Description("Specifies a number to be raised to a power.")]
        public double A;
        [DataMember]
        [Description("Specifies a power.")]
        public double Exponent;
    }

    [DataContract]
    public class RandomRequest
    {
    }

    [DataContract]
    public class RoundRequest
    {
        [DataMember]
        [Description("Specifies a decimal number to be rounded.")]
        public double Value;
        [DataMember]
        [Description("Specifies the number of significant decimal places (precision) in the return value.")]
        public int Digits;
    }

    [DataContract]
    public class SinRequest
    {
        [DataMember]
        [Description("Specifies an angle, measured in radians.")]
        public double Angle;
    }

    [DataContract]
    public class SqrtRequest
    {
        [DataMember]
        [Description("Specifies a number to calculate its square root.")]
        public double A;
    }

    [DataContract]
    public class TanRequest
    {
        [DataMember]
        [Description("Specifies an angle, measured in radians.")]
        public double Angle;
    }

    [DataContract]
    public class TruncateRequest
    {
        [DataMember]
        [Description("Specifies a number to truncate.")]
        public double A;
    }

    [DataContract]
    public class ToRadiansRequest
    {
        [DataMember]
        [Description("Specifies an angle, measured in degrees.")]
        public double Angle;
    }

    [DataContract]
    public class ToDegreesRequest
    {
        [DataMember]
        [Description("Specifies an angle, measured in radians.")]
        public double Angle;
    }

    [ServicePort]
    public class MathOperations : PortSet
    {
        public MathOperations()
            : base(
                typeof(DsspDefaultLookup), 
                typeof(Constants), 
                typeof(Acos), 
                typeof(Asin), 
                typeof(Atan), 
                typeof(Atan2), 
                typeof(Cos), 
                typeof(Exp), 
                typeof(Log), 
                typeof(Pow), 
                typeof(Random), 
                typeof(Round), 
                typeof(Sin), 
                typeof(Sqrt), 
                typeof(Tan), 
                typeof(Truncate), 
                typeof(ToRadians), 
                typeof(ToDegrees)
            )
        {
        }

    }

    [DisplayName("PiandE")]
    [Description("Returns the values of the mathematical constants PI and E.")]
    public class Constants : Get<GetRequestType, PortSet<MathConstants, Fault>>
    {
    }

    [DisplayName("ArcCosine")]
    [Description("Returns the angle (in radians) whose cosine is the specified number.")]
    public class Acos : Submit<AcosRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("ArcSine")]
    [Description("Returns the angle (in radians) whose sine is the specified number.")]
    public class Asin : Submit<AsinRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("ArcTangent")]
    [Description("Returns the angle (in radians) whose tangent is the specified number.")]
    public class Atan : Submit<AtanRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("ArcTangent2")]
    [Description("Returns the angle (in radians) whose tangent is the quotient of two specified numbers.")]
    public class Atan2 : Submit<Atan2Request, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Cosine")]
    [Description("Returns the cosine of the specified angle (in radians).")]
    public class Cos : Submit<CosRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Exponent")]
    [Description("Returns e raised to the specified power.")]
    public class Exp : Submit<ExpRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Logarithm")]
    [Description("Returns the logarithm of a specified number. If base is specified as 0 then a natural logarithm (base e) is calculated.")]
    public class Log : Submit<LogRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Power")]
    [Description("Raises 'A' to the power 'Exponent'.")]
    public class Pow : Submit<PowRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Random")]
    [Description("Returns a psuedo-random number between 0.0 and 1.0.")]
    public class Random : Query<RandomRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Round")]
    [Description("Rounds a double-precision floating-point value to the specified precision.")]
    public class Round : Submit<RoundRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Sine")]
    [Description("Returns the sine of the specified angle (in radians).")]
    public class Sin : Submit<SinRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Square Root")]
    [Description("Returns the square root of the specified number.")]
    public class Sqrt : Submit<SqrtRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Tangent")]
    [Description("Returns the tangent of the specified angle (in radians).")]
    public class Tan : Submit<TanRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("Truncate")]
    [Description("Calculates the integral part of a specified double-precision floating-point number.")]
    public class Truncate : Submit<TruncateRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("ToRadians")]
    [Description("Converts a value in degrees to radians.")]
    public class ToRadians : Submit<ToRadiansRequest, PortSet<Response, Fault>>
    {
    }

    [DisplayName("ToDegrees")]
    [Description("Converts a value in radians to degrees.")]
    public class ToDegrees : Submit<ToDegreesRequest, PortSet<Response, Fault>>
    {
    }

}
