//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: TimerTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using W3C.Soap;

using timer = Microsoft.Robotics.Services.Sample.Timer;

namespace Microsoft.Robotics.Services.Sample.Timer
{
    /// Timer Contract
    public static class Contract
    {
        /// The Unique Contract Identifier for the Timer service
        public const string Identifier = "http://schemas.microsoft.com/robotics/2006/08/timer.html";
    }
    [DataContract()]
    public class TimerState
    {
        private int _timeout;
        [DataMember]
        [Description("Indicates the timeout interval (in ms).")]
        public int Timeout
        {
            get { return _timeout; }
            set { _timeout = value; }
        }

        private DateTime _expires = DateTime.MaxValue;
        [DataMember]
        [Description("Indicates when the timeout interval expires.")]
        public DateTime Expires
        {
            get { return _expires; }
            set { _expires = value; }
        }

        private int _ticks;

        public int Ticks
        {
            get { return _ticks; }
            set { _ticks = value; }
        }
	
    }

    [DataContract]
    public class SetTimerRequest
    {
        int _interval;
        [DataMember]
        [Description("Specifies the timeout interval setting (in ms).")]
        public int Interval
        {
            get { return _interval; }
            set { _interval = value; }
        }
    }

    [DataContract]
    public class FireTimerRequest : SetTimerRequest
    {
        private DateTime _fired;
        [DataMember]
        [Description("Indicates the time at which the timer fired.")]
        public DateTime Fired
        {
            get { return _fired; }
            set { _fired = value; }
        }
    }

    [DataContract]
    public class TickRequest
    {
    }

    [DataContract]
    public class TimeRequest
    {
    }

    [DataContract]
    public class TimeResponse
    {
        public TimeResponse()
        {
        }

        public TimeResponse(DateTime time)
        {
            _time = time;
        }

        private DateTime _time;
        [DataMember]
        [Description("Indicates the current time.")]
        public DateTime Time
        {
            get { return _time; }
            set { _time = value; }
        }
	
    }

    [ServicePort]
    public class TimerOperations : PortSet
    {
        public TimerOperations()
            : base(
                typeof(DsspDefaultLookup),
                typeof(DsspDefaultDrop),
                typeof(Get),
                typeof(FireTimer),
                typeof(SetTimer),
                typeof(Tick),
                typeof(Wait),
                typeof(Subscribe),
                typeof(Time)
            )
        {
        }

        public static implicit operator Port<FireTimer>(TimerOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (Port<FireTimer>)portSet[typeof(FireTimer)];
        }

        public void Post(FireTimer msg)
        {
            base.PostUnknownType(msg);
        }

        public static implicit operator Port<Tick>(TimerOperations portSet)
        {
            if (portSet == null)
            {
                return null;
            }
            return (Port<Tick>)portSet[typeof(Tick)];
        }

        public void Post(Tick msg)
        {
            base.PostUnknownType(msg);
        }
    }

    [Description("Get the timer's current state.\nThis includes the currently set timeout value (if any) and the expected time at which it will fire.")]
    public class Get : Get<GetRequestType, PortSet<TimerState, Fault>>
    {
    }

    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }
    [Description("Set the timer in milliseconds or indicates the timer has been set.\nThe TimerComplete notification is sent after that interval has passed\nSetting a timer before the previous timer has fired superceds the previous timer.\nSetting a timer to 0 will stop any existing timer.")]
    public class SetTimer : Update<SetTimerRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    [Description("Sends a notification every second.")]
    public class Tick : Update<TickRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public Tick()
        {
        }

        public Tick(TickRequest body)
            : base(body)
        {
        }
    }

    [DisplayName("TimerComplete")]
    [Description("Indicates that the timer (set with SetTimer) has elapsed.")]
    public class FireTimer : Update<FireTimerRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public FireTimer()
        {
        }

        public FireTimer(FireTimerRequest body)
            : base(body)
        {
        }
    }

    [Description("Wait for the specified interval in milliseconds.\nNote: This is intended for intervals of less than 60 seconds only,\nfor longer intervals use SetTimer.")]
    public class Wait : Submit<SetTimerRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
    }

    [DisplayName("GetCurrentTime")]
    [Description("Returns the current time.")]
    public class Time : Submit<TimeRequest, PortSet<TimeResponse, Fault>>
    {
    }
}
