//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: WinMessageTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------

using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using System;
using System.Collections.Generic;
using W3C.Soap;
using winmessage = Microsoft.Robotics.Services.Sample.WinMessage;
using System.ComponentModel;


namespace Microsoft.Robotics.Services.Sample.WinMessage
{
    
    /// <summary>
    /// WinMessage Contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// The Dss Service contract
        /// </summary>
        public const String Identifier = "http://schemas.microsoft.com/robotics/2007/05/winmessage.html";
    }

    /// <summary>
    /// The WinMessage State
    /// </summary>
    [DataContract()]
    [Description("Identifies the state of the Windows Messages Sample service.")]
    public class WinMessageState
    {
        /// <summary>
        /// The Windows Title which is capable of receiving messages.
        /// </summary>
        [DataMember, DataMemberConstructor(Order=1)]
        [Description("Identifies the Windows Process Name capable of receiving messages.")]
        public string ProcessName;

        /// <summary>
        /// Show the Message Window
        /// </summary>
        [DataMember, DataMemberConstructor(Order = 2)]
        [Description("Indicates whether to show the Windows Messages Sample Form.")]
        public bool ShowWindow;
    }


    /// <summary>
    /// WinMessage Main Operations Port
    /// </summary>
    [ServicePort]
    public class WinMessageOperations : PortSet<
        DsspDefaultLookup,
        DsspDefaultDrop,
        Get,
        Configure,
        Subscribe,
        ReceiveMessage,
        SendMessage,
        SendMessageString,
        PlaySound
        >
    {
    }

    /// <summary>
    /// WinMessage Get Operation
    /// </summary>
    [Description("Get the Windows message service state.")]
    public class Get : Get<GetRequestType, PortSet<WinMessageState, Fault>>
    {
    }

    /// <summary>
    /// Subscribe Operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    /// <summary>
    /// ReceiveWinMsg Operation
    /// </summary>
    [Description("Receives a Windows message notification.")]
    public class ReceiveMessage : Update<WinMsgReceived, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public ReceiveMessage()
        {
        }
        /// <summary>
        /// Constructor with WinMsgReceived
        /// </summary>
        public ReceiveMessage(WinMsgReceived body)
            : base(body)
        {
        }
    }

    /// <summary>
    /// SendWinMsg Operation
    /// </summary>
    [Description("Sends a Windows message.\nThis opens sends the specified message to a window or windows, calling the window procedure for the specified window, and does not return until the window procedure has processed the message.")]
    public class SendMessage : Update<WinMsg, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public SendMessage()
        {
        }
        /// <summary>
        /// Constructor with WinMsg
        /// </summary>
        public SendMessage(WinMsg body)
            : base(body)
        {
        }
    }

    /// <summary>
    /// SendWinMsgCopyData Operation
    /// </summary>
    [Description("Send a windows message to pass data to another application.")]
    public class SendMessageString : Update<WinMsgCopyData, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public SendMessageString()
        {
        }
        /// <summary>
        /// Constructor with WinMsgCopyData
        /// </summary>
        public SendMessageString(WinMsgCopyData body)
            : base(body)
        {
        }
    }

    /// <summary>
    /// Configure the WinMessage Service
    /// </summary>
    [Description("Configure the Windows Messages Sample Service")]
    public class Configure : Replace<WinMessageState, PortSet<DefaultReplaceResponseType, Fault>>
    {
        /// <summary>
        /// Configure the WinMessage Service
        /// </summary>
        public Configure()
        {
        }
        /// <summary>
        /// Configure the WinMessage Service
        /// </summary>
        public Configure(WinMessageState body)
            : base(body)
        {
        }
    }


    /// <summary>
    /// PlaySound Operation
    /// </summary>
    [Description("Plays a sound file.")]
    public class PlaySound : Update<PlaySoundRequest, PortSet<DefaultUpdateResponseType, Fault>>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public PlaySound()
        {
        }
        /// <summary>
        /// Constructor with PlaySoundRequest
        /// </summary>
        public PlaySound(PlaySoundRequest body)
            : base(body)
        {
        }

        /// <summary>
        /// Constructor with PlaySoundRequest
        /// </summary>
        public PlaySound(string filename)
        {
            this.Body = new PlaySoundRequest(filename);
        }
    }

    /// <summary>
    /// WinMsgReceived
    /// </summary>
    [DataContract]
    public class WinMsgReceived
    {
        string _processName;
        string _stringData;
        int _msg;
        int _wParam;
        int _lParam;

        /// <summary>
        /// ProcessName
        /// </summary>
        [DataMember]
        [Description("Identifies the Windows process name.")]
        public string ProcessName
        {
            get { return _processName; }
            set { _processName = value; }
        }
        /// <summary>
        /// StringData
        /// </summary>
        [DataMember]
        [Description ("Indicates the string data received.")]
        public string StringData
        {
            get { return _stringData; }
            set { _stringData = value; }
        }
        /// <summary>
        /// Msg
        /// </summary>
        [DataMember]
        [Description("Indicates the message received.")]
        public int Msg
        {
            get { return _msg; }
            set { _msg = value; }
        }
        /// <summary>
        /// WParam
        /// </summary>
        [DataMember]
        [DisplayName("wParam")]
        [Description("Identifies additional message specific information.")]
        public int WParam
        {
            get { return _wParam; }
            set { _wParam = value; }
        }
        /// <summary>
        /// LParam
        /// </summary>
        [DataMember]
        [DisplayName("lParam")]
        [Description("Identifies additional message specific information.")]
        public int LParam
        {
            get { return _lParam; }
            set { _lParam = value; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public WinMsgReceived()
        {
        }
        /// <summary>
        /// Constructor with processName, msg, wParam, data
        /// </summary>
        public WinMsgReceived(String processName, int msg, int wParam, String data)
        {
            _processName = processName;
            _msg = msg;
            _wParam = wParam;
            _stringData = data;
        }
        /// <summary>
        /// Constructor with processName, msg, wParam, lParam
        /// </summary>
        public WinMsgReceived(String processName, int msg, int wParam, int lParam)
        {
            _processName = processName;
            _msg = msg;
            _wParam = wParam;
            _lParam = lParam;
        }
    }

    /// <summary>
    /// WinMsg
    /// </summary>
    [DataContract]
    [Description("The SendMessage operation sends the specified message to a window or windows, calling the window procedure for the specified window, and does not return until the window procedure has processed the message.")]
    public class WinMsg
    {
        string _processName;
        int _msg;
        int _wParam;
        int _lParam;

        /// <summary>
        /// ProcessName
        /// </summary>
        [DataMember]
        [Description("Identifies the Windows process name.")]
        public string ProcessName
        {
            get { return _processName; }
            set { _processName = value; }
        }
        /// <summary>
        /// Msg
        /// </summary>
        [DataMember]
        [Description("Specifies the message to be sent.")]
        public int Msg
        {
            get { return _msg; }
            set { _msg = value; }
        }
        /// <summary>
        /// WParam
        /// </summary>
        [DataMember]
        [DisplayName("wParam")]
        [Description("Specifies additional message specific information.")]
        public int WParam
        {
            get { return _wParam; }
            set { _wParam = value; }
        }
        /// <summary>
        /// LParam
        /// </summary>
        [DataMember]
        [DisplayName("lParam")]
        [Description("Specifies additional message specific information.")]
        public int LParam
        {
            get { return _lParam; }
            set { _lParam = value; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public WinMsg()
        {
        }
        /// <summary>
        /// Constructor with processName, msg, wParamn, lParam
        /// </summary>
        public WinMsg(String processName, int msg, int wParam, int lParam)
        {
            _processName = processName;
            _msg = msg;
            _wParam = wParam;
            _lParam = lParam;
        }
    }

    /// <summary>
    /// WinMsgCopyData
    /// </summary>
    [DataContract]
    public class WinMsgCopyData
    {
        string _processName;
        string _stringData;
        int _wParam;

        /// <summary>
        /// ProcessName
        /// </summary>
        [DataMember, DataMemberConstructor(Order=1)]
        [Description("Identifies the name of the Windows process.\nUses HWND_BROADCAST to broadcast the message")]
        public string ProcessName
        {
            get { return _processName; }
            set { _processName = value; }
        }
        /// <summary>
        /// StringData
        /// </summary>
        [DataMember, DataMemberConstructor(Order = 3)]
        [Description("Identifies the data to send.")]
        public string StringData
        {
            get { return _stringData; }
            set { _stringData = value; }
        }
        /// <summary>
        /// WParam
        /// </summary>
        [DisplayName("wParam")]
        [Description("Specifies additional message specific information.")]
        public int WParam
        {
            get { return _wParam; }
            set { _wParam = value; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public WinMsgCopyData()
        {
        }
        /// <summary>
        /// Constructor with processName, wParam, data
        /// </summary>
        public WinMsgCopyData(String processName, int wParam, String data)
        {
            _processName = processName;
            _wParam = wParam;
            _stringData = data;
        }
    }

    [DataContract]
    [Flags]
    [Description("Specifies how to play the sound file.")]
    public enum PlaySoundFlags
    {
        /// <summary>
        /// Play synchronously
        /// </summary>
        [Description("Specifies how to play the sound file.")]
        Default = 0x0000,

        /// <summary>
        /// Play asynchronously
        /// </summary>
        SND_ASYNC = 0x0001,
        SND_NODEFAULT = 0x0002,
        /// <summary>
        /// Loop the sound until the next PlaySound command
        /// </summary>
        SND_LOOP = 0x0008,
        /// <summary>
        /// Don't stop any currently playing sound.  If a sound is currently playing, the function immediately returns FALSE, without playing the requested sound. 
        /// </summary>
        SND_NOSTOP = 0x0010,
        /// <summary>
        /// Don't wait if the driver is busy
        /// </summary>
        SND_NOWAIT = 0x00002000,
        /// <summary>
        /// The name is a filename
        /// </summary>
        SND_FILENAME = 0x00020000,
        /// <summary>
        /// The name is a resource name or atom
        /// </summary>
        SND_RESOURCE = 0x00040004
    }

    /// <summary>
    /// PlaySoundRequest
    /// </summary>
    [DataContract]
    public class PlaySoundRequest
    {
        string _fileName;
        PlaySoundFlags _soundFlags;

        /// <summary>
        /// FileName
        /// </summary>
        [DataMember, DataMemberConstructor(Order=1)]
        [Description("Specifies the name of the sound file.")]
        public string FileName
        {
            get { return _fileName; }
            set { _fileName = value; }
        }

        /// <summary>
        /// FileName
        /// </summary>
        [DataMember]
        [Description("Specifies flags for playing the sound file.")]
        public PlaySoundFlags Flags
        {
            get { return _soundFlags; }
            set { _soundFlags = value; }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public PlaySoundRequest()
        {
        }
        /// <summary>
        /// Constructor with filename
        /// </summary>
        public PlaySoundRequest(String filename)
        {
            _fileName = filename;
        }
    }
}
