using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;
using Microsoft.Ccr.Core;

namespace Microsoft.Robotics.Services.Sample.WinMessage
{
    partial class WinMsgForm : Form
    {
        // Define a special delegate that handles marshaling
        // form configuration data to the UI.
        private delegate void ConfigurationDelegate(WinMessageState config);
        private delegate void SetStringDelegate(string text);

        private Port<bool> _initializedPort = null;
        private bool initialized = false;

        /*      
        // referred from "winuser.h"
        public const int WM_SYSCOMMAND = 0x0112;
        public const int WM_COPYDATA = 0x004A;
        public const int WM_CLOSE = 0x0010;
        public const int WM_QUIT = 0x0012;
        public const int WM_CREATE = 0x0001;
        public const int WM_DESTROY = 0x0002;
        public const int WM_MOVE = 0x0003;
        public const int WM_SIZE = 0x0005;

        public const int WM_MOUSEFIRST = 0x0200;
        public const int WM_MOUSEMOVE = 0x0200;
        public const int WM_LBUTTONDOWN = 0x0201;
        public const int WM_LBUTTONUP = 0x0202;
        public const int WM_LBUTTONDBLCLK = 0x0203;
        public const int WM_RBUTTONDOWN = 0x0204;
        public const int WM_RBUTTONUP = 0x0205;
        public const int WM_RBUTTONDBLCLK = 0x0206;
        public const int WM_MBUTTONDOWN = 0x0207;
        public const int WM_MBUTTONUP = 0x0208;
        public const int WM_MBUTTONDBLCLK = 0x0209;
        */
        WinMessageOperations _mainPort;
        WinMessageOperations _internalPort;
        WinMessageState _config = null;
        bool _configChanged = false;

        /// <summary>
        /// Initialize the WinMsgForm with information about the WinMessage service.
        /// </summary>
        /// <param name="mainPort"></param>
        /// <param name="internalPort"></param>
        /// <param name="title"></param>
        public WinMsgForm(WinMessageOperations mainPort, WinMessageOperations internalPort, WinMessageState config, Port<bool> initPort)
        {
            InitializeComponent();

            _mainPort = mainPort;
            _internalPort = internalPort;
            _initializedPort = initPort;

            SetConfiguration(config);
        }

        public void SetConfiguration(WinMessageState config)
        {
            // InvokeRequired compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.textBoxProcessName.InvokeRequired)
            {
                ConfigurationDelegate d = new ConfigurationDelegate(SetConfiguration);
                this.Invoke(d, new object[] { config });
            }
            else
            {
                _config = config;
                if (initialized)
                    UpdateUI();
            }
        }

        public void SetStatus(string text)
        {
            // InvokeRequired compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.textBoxProcessName.InvokeRequired)
            {
                SetStringDelegate d = new SetStringDelegate(SetStatus);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                statusStrip1.Items[0].Text = text;
            }
        }


        /// <summary> 
        /// Structure required to be sent with the received WM_COPYDATA message 
        /// </summary> 
        [StructLayout(LayoutKind.Sequential)]
        struct COPYDATASTRUCT
        {
            public int dwData;
            public int cbData;
            public int lpData;
        }

        // Window proc that receives the WM_ messages for the associated window handle
        protected override void WndProc(ref Message m)
        {
            const int WM_COPYDATA = 0x004A;

            switch (m.Msg)
            {
                case WM_COPYDATA:
                    COPYDATASTRUCT CD =
                    (COPYDATASTRUCT)m.GetLParam(typeof(COPYDATASTRUCT));
                    byte[] B = new byte[CD.cbData];
                    IntPtr lpData = new IntPtr(CD.lpData);
                    Marshal.Copy(lpData, B, 0, CD.cbData);
                    string strData = Encoding.Default.GetString(B).Trim((char)0);

                    // posts the received WM_COPYDATA with string to _mainPort
                    _internalPort.Post(new ReceiveMessage(new WinMsgReceived(m.HWnd.ToString(), m.Msg, (int)m.WParam, strData)));

                    SetStatus(strData);
                    break;

                // add other win messages that you want to receive
                default:
                    //_internalPort.Post(new ReceiveWinMsg(new WinMsgReceived(m.HWnd.ToString(), m.Msg, (int)m.WParam,  (int)m.LParam)));
                    break;
            }

            base.WndProc(ref m);
        }

        private void buttonSendData_Click(object sender, EventArgs e)
        {
            // send window message WM_COPYDATA with TextBox.Text string to 'TEST' process 
            _mainPort.Post(new SendMessageString(new WinMsgCopyData(textBoxSendToProcessName.Text, 0, textBoxStringToSend.Text)));
        }

        private void buttonPlaySound_Click(object sender, EventArgs e)
        {
            // PlaySound 
            _mainPort.Post(new PlaySound(new PlaySoundRequest(textBoxStringToSend.Text)));
        }

        private void WinMsgForm_Load(object sender, EventArgs e)
        {
            initialized = true;
            _initializedPort.Post(true);
            UpdateUI();
        }

        private void UpdateUI()
        {
            if (_config != null)
            {
                if (_config.ShowWindow == checkBoxHide.Checked)
                    checkBoxHide.Checked = !_config.ShowWindow;

                if (!string.IsNullOrEmpty(_config.ProcessName) && textBoxProcessName.Text != _config.ProcessName)
                    textBoxProcessName.Text = _config.ProcessName;

                this.Visible = _config.ShowWindow;
                this.ShowInTaskbar = _config.ShowWindow;
                
                if (this.Text != textBoxProcessName.Text)
                    this.Text = textBoxProcessName.Text;

                if (_config.ShowWindow)
                {
                    this.WindowState = FormWindowState.Normal;
                }

            }
        }

        private void textBoxProcessName_TextChanged(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(textBoxProcessName.Text))
                this.Text = textBoxProcessName.Text;
            _configChanged = true;
            UpdateConfiguration();
        }

        private void textBoxProcessName_Leave(object sender, EventArgs e)
        {
            UpdateConfiguration();
        }

        private void UpdateConfiguration()
        {
            if (_configChanged)
            {
                WinMessageState config = new WinMessageState();
                config.ProcessName = textBoxProcessName.Text;
                config.ShowWindow = !checkBoxHide.Checked;
                this.Visible = config.ShowWindow;
                _internalPort.Post(new Configure(config));
                _configChanged = false;
            }
        }

        private void checkBoxHide_CheckedChanged(object sender, EventArgs e)
        {
            _configChanged = true;
            UpdateConfiguration();
        }


    }
}