//-----------------------------------------------------------------------
//  This file is part of the Microsoft Robotics Studio Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  $File: XInputGamepadTypes.cs $ $Revision: 1 $
//-----------------------------------------------------------------------
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.Core.DsspHttp;

using System;
using System.Net;
using System.Collections.Generic;
using System.ComponentModel;
using W3C.Soap;

using xinput = Microsoft.Xna.Framework.Input;
using xna = Microsoft.Xna.Framework;


namespace Microsoft.Robotics.Services.Sample.XInputGamepad
{
    /// <summary>
    /// Xinput Contract
    /// </summary>
    public static class Contract
    {
        /// The Unique Contract Identifier for the Xinput service
        public const String Identifier = "http://schemas.microsoft.com/robotics/2006/09/xinputgamepad.html";
    }


    [DataContract]
    [Description("Specifies the state of the XInput controller.")]
    public class XInputGamepadState
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Browsable(false)]
        [Description("Indicates time of the controller's state update.")]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        private Controller _controller = new Controller();
        [DataMember(IsRequired = true)]
        [Description("Indicates controller state.")]
        public Controller Controller
        {
            get { return _controller; }
            set { _controller = value; }
        }

        private DPad _dPad = new DPad();
        [DataMember(IsRequired = true)]
        [Description("Indicates the state of the directional pad.")]
        [Browsable(false)]
        public DPad DPad
        {
            get { return _dPad; }
            set { _dPad = value; }
        }

        private Buttons _buttons = new Buttons();
        [DataMember(IsRequired = true)]
        [Description("Identifies the state of the buttons.")]
        [Browsable(false)]
        public Buttons Buttons
        {
            get { return _buttons; }
            set { _buttons = value; }
        }

        private Triggers _triggers = new Triggers();
        [DataMember(IsRequired = true)]
        [Description ("Identifies the state of the triggers.")]
        [Browsable(false)]
        public Triggers Triggers
        {
            get { return _triggers; }
            set { _triggers = value; }
        }
	

        private Thumbsticks _thumbsticks = new Thumbsticks();
        [DataMember(IsRequired = true)]
        [Description("Identifies the state of the thumbsticks.")]
        [Browsable(false)]
        public Thumbsticks Thumbsticks
        {
            get { return _thumbsticks; }
            set { _thumbsticks = value; }
        }


        private Vibration _vibration = new Vibration();
        [DataMember(IsRequired = true)]
        [Description("Identifies the state of the vibration motors.")]
        [Browsable(false)]
        public Vibration Vibration
        {
            get { return _vibration; }
            set { _vibration = value; }
        }
	

        private xinput.GamePadState _previous = new xinput.GamePadState();

        internal xinput.GamePadState GetState()
        {
            return xinput.GamePad.GetState(
                (xna.PlayerIndex)_controller.PlayerIndex
            );
        }

        internal bool Update(DateTime timeStamp, xinput.GamePadState state)
        {
            if (_previous == state)
            {
                return false;
            }

            _timeStamp = timeStamp;

            _previous = state;
            return true;
        }

        internal void SetVibration(Vibration vibration)
        {
            vibration.Left = Math.Max(0, Math.Min(vibration.Left, 1.0f));
            vibration.Right = Math.Max(0, Math.Min(vibration.Right, 1.0f));
            vibration.TimeStamp = DateTime.Now;

            xinput.GamePad.SetVibration(
                (xna.PlayerIndex)_controller.PlayerIndex,
                vibration.Left,
                vibration.Right
            );

            _vibration = vibration;
        }
    }

    [DataContract]
    [Description ("Identifies the player index.")]
    [Browsable(false)]
    public enum PlayerIndex
    {
        One = 0,
        Two = 1,
        Three = 2,
        Four = 3,
        Any = 255
    }

    [DataContract]
    [Description ("Identifies the controller type.")]
    [Browsable(false)]
    public enum ControllerType
    {
        NotSelected = 0,
        GamePad = 1,
        Wheel = 2,
        ArcadeStick = 3,
        FlightStick = 4,
        DancePad = 5,
    }

    [DataContract]
    public class Controller
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description("Indicates the time of the controller change.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        private PlayerIndex _playerIndex = PlayerIndex.One;
        [DataMember]
        [Description("Identifies the player index.")]
        public PlayerIndex PlayerIndex
        {
            get { return _playerIndex; }
            set { _playerIndex = value; }
        }

        private bool _isConnected;
        [DataMember]
        [Description ("Indicates whether a controller is connected.")]
        [Browsable(false)]
        public bool IsConnected
        {
            get { return _isConnected; }
            set { _isConnected = value; }
        }

        internal bool Update(xinput.GamePadState curr)
        {
            if (_isConnected == curr.IsConnected)
            {
                return false;
            }

            _isConnected = curr.IsConnected;

            return true;
        }

        public bool IsValid
        {
            get
            {
                return _playerIndex >= PlayerIndex.One &&
               _playerIndex <= PlayerIndex.Four;
            }
        }
    }

    [DataContract]
    public class ControllerCaps : Controller
    {
        private ControllerType _ControllerType;
        [DataMember]
        public ControllerType ControllerType
        {
            get { return _ControllerType; }
            set { _ControllerType = value; }
        }
	
    }

    [DataContract]
    [Description("Identifies the state of the directional pad.")]
    public class DPad
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description ("Identifies the time of the directional pad input.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }
	
        private bool _left;
        [DataMember]
        [Description("Indicates if the directional pad is pressed left.")]
        [Browsable(false)]
        public bool Left
        {
            get { return _left; }
            set { _left = value; }
        }

        private bool _down;
        [DataMember]
        [Description("Indicates if the directional pad is pressed down.")]
        [Browsable(false)]
        public bool Down
        {
            get { return _down; }
            set { _down = value; }
        }

        private bool _right;
        [DataMember]
        [Description("Indicates if the directional pad is pressed right.")]
        [Browsable(false)]
        public bool Right
        {
            get { return _right; }
            set { _right = value; }
        }

        private bool _up;
        [DataMember]
        [Description("Indicates if the directional pad is pressed up.")]
        [Browsable(false)]
        public bool Up
        {
            get { return _up; }
            set { _up = value; }
        }

        xinput.GamePadDPad _previous = new xinput.GamePadDPad();

        internal bool Update(DateTime timeStamp, xinput.GamePadDPad curr)
        {
            if (_previous == curr)
            {
                return false;
            }

            _timeStamp = timeStamp;

            _left = (curr.Left == xinput.ButtonState.Pressed);
            _down = (curr.Down == xinput.ButtonState.Pressed);
            _right = (curr.Right == xinput.ButtonState.Pressed);
            _up = (curr.Up == xinput.ButtonState.Pressed);

            _previous = curr;

            return true;
        }
    }

    [DataContract]
    [Description("Identifies the state of the controller buttons.")]
    [Browsable(false)]
    public class Buttons
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description("Identifies the time of the button input.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        bool _a;
        [DataMember]
        [Description("Identifies whether the A button was pressed.")]
        [Browsable(false)]
        public bool A
        {
            get { return _a; }
            set { _a = value; }
        }

        bool _b;
        [DataMember]
        [Description("Identifies whether the B button was pressed.")]
        [Browsable(false)]
        public bool B
        {
            get { return _b; }
            set { _b = value; }
        }

        bool _x;
        [DataMember]
        [Description("Identifies whether the X button was pressed.")]
        [Browsable(false)]
        public bool X
        {
            get { return _x; }
            set { _x = value; }
        }

        bool _y;
        [DataMember]
        [Description("Identifies whether the Y button was pressed.")]
        [Browsable(false)]
        public bool Y
        {
            get { return _y; }
            set { _y = value; }
        }

        bool _back;
        [DataMember]
        [Description("Identifies whether the Back button was pressed.")]
        [Browsable(false)]
        public bool Back
        {
            get { return _back; }
            set { _back = value; }
        }

        bool _start;
        [DataMember]
        [Description("Identifies whether the Start button was pressed.")]
        [Browsable(false)]
        public bool Start
        {
            get { return _start; }
            set { _start = value; }
        }

        bool _leftStick;
        [DataMember]
        [Description("Identifies whether the left thumbstick was pressed.")]
        [Browsable(false)]
        public bool LeftStick
        {
            get { return _leftStick; }
            set { _leftStick = value; }
        }

        bool _rightStick;
        [DataMember]
        [Description("Identifies whether the right thumbstick was pressed.")]
        [Browsable(false)]
        public bool RightStick
        {
            get { return _rightStick; }
            set { _rightStick = value; }
        }

        bool _leftShoulder;
        [DataMember]
        [Description("Identifies whether the left shoulder (bumper) was pressed.")]
        [Browsable(false)]
        public bool LeftShoulder
        {
            get { return _leftShoulder; }
            set { _leftShoulder = value; }
        }

        bool _rightShoulder;
        [DataMember]
        [Description("Identifies whether the right shoulder (bumper) was pressed.")]
        [Browsable(false)]
        public bool RightShoulder
        {
            get { return _rightShoulder; }
            set { _rightShoulder = value; }
        }

        xinput.GamePadButtons _previous = new xinput.GamePadButtons();

        internal bool Update(DateTime timeStamp, xinput.GamePadButtons curr)
        {
            if (curr == _previous)
            {
                return false;
            }

            _timeStamp = timeStamp;

            _a = (curr.A == xinput.ButtonState.Pressed);
            _b = (curr.B == xinput.ButtonState.Pressed);
            _x = (curr.X == xinput.ButtonState.Pressed);
            _y = (curr.Y == xinput.ButtonState.Pressed);

            _leftStick = (curr.LeftStick == xinput.ButtonState.Pressed);
            _rightStick = (curr.RightStick == xinput.ButtonState.Pressed);
            _leftShoulder = (curr.LeftShoulder == xinput.ButtonState.Pressed);
            _rightShoulder = (curr.RightShoulder == xinput.ButtonState.Pressed);

            _back = (curr.Back == xinput.ButtonState.Pressed);
            _start = (curr.Start == xinput.ButtonState.Pressed);

            _previous = curr;

            return true;
        }
    }

    [DataContract]
    [Description("Identifies the state of the controller's triggers.")]
    [Browsable(false)]
    public class Triggers
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description("Identifies the time of the trigger input.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        float _left;
        [DataMember]
        [Description("Identifies the left trigger value.")]
        [Browsable(false)]
        public float Left
        {
            get { return _left; }
            set { _left = value; }
        }

        float _right;
        [DataMember]
        [Description("Identifies the right trigger value.")]
        [Browsable(false)]
        public float Right
        {
            get { return _right; }
            set { _right = value; }
        }

        xinput.GamePadTriggers _previous = new xinput.GamePadTriggers();

        internal bool Update(DateTime timeStamp, xinput.GamePadTriggers curr)
        {
            if (_left == curr.Left &&
                _right == curr.Right)
            {
                return false;
            }

            _timeStamp = timeStamp;

            _left = curr.Left;
            _right = curr.Right;

            _previous = curr;
            return true;
        }
    }
    
    [DataContract]
    [Description("Identifies the thumbsticks state.")]
    [Browsable(false)]
    public class Thumbsticks
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description("Identifies the time of the thumbstick input.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }

        float _leftX;
        [DataMember]
        [Description("Identifies the X value of left thumbstick.")]
        [Browsable(false)]
        public float LeftX
        {
            get { return _leftX; }
            set { _leftX = value; }
        }

        float _leftY;
        [DataMember]
        [Description("Identifies the Y value of left thumbstick.")]
        [Browsable(false)]
        public float LeftY
        {
            get { return _leftY; }
            set { _leftY = value; }
        }

        float _rightX;
        [DataMember]
        [Description("Identifies the X value of right thumbstick.")]
        [Browsable(false)]
       public float RightX
        {
            get { return _rightX; }
            set { _rightX = value; }
        }

        float _rightY;
        [DataMember]
        [Description("Identifies the Y value of right thumbstick.")]
        [Browsable(false)]
        public float RightY
        {
            get { return _rightY; }
            set { _rightY = value; }
        }

        xinput.GamePadThumbSticks _previous = new xinput.GamePadThumbSticks();

        internal bool Update(DateTime timeStamp, xinput.GamePadThumbSticks curr)
        {
            if (_leftX == curr.Left.X &&
                _leftY == curr.Left.Y &&
                _rightX == curr.Right.X &&
                _rightY == curr.Right.Y)
            {
                return false;
            }

            _timeStamp = timeStamp;

            _leftX = curr.Left.X;
            _leftY = curr.Left.Y;
            _rightX = curr.Right.X;
            _rightY = curr.Right.Y;

            _previous = curr;

            return true;
        }
    }

    [DataContract]
    public class PollRequest
    {
    }

    [DataContract]
    [Description("Identifies the vibration motors' state.")]
    [Browsable(false)]
    public class Vibration
    {
        private DateTime _timeStamp = DateTime.Now;
        [DataMember]
        [Description("Identifies the time of the vibration motors change.")]
        [Browsable(false)]
        public DateTime TimeStamp
        {
            get { return _timeStamp; }
            set { _timeStamp = value; }
        }
	
        private float _left;
        [DataMember]
        [Description("Identifies the value of the left vibration motor.")]
        [Browsable(false)]
        public float Left
        {
            get { return _left; }
            set { _left = value; }
        }

        private float _right;
        [DataMember]
        [Description("Identifies the value of the right vibration motor.")]
        [Browsable(false)]
        public float Right
        {
            get { return _right; }
            set { _right = value; }
        }
    }

    [DataContract]
    public class QueryControllersRequest
    {
    }

    [DataContract]
    public class QueryControllersResponse
    {
        private List<ControllerCaps> _controllers = new List<ControllerCaps>();
        [DataMember(IsRequired = true)]
        public List<ControllerCaps> Controllers
        {
            get { return _controllers; }
            set { _controllers = value; }
        }
    }

    [ServicePort]
    public class XInputGamepadOperations : PortSet<DsspDefaultLookup, DsspDefaultDrop, Subscribe, Get, HttpGet, Replace, Poll, DPadChanged, ButtonsChanged, TriggersChanged, ThumbsticksChanged, ControllerChanged, SetVibration, QueryControllers>
    {
    }

    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    [Description("Gets the current state of the controller.")]
    public class Get : Get<GetRequestType, PortSet<XInputGamepadState, Fault>>
    {
    }

    /// <summary>
    /// Sets the current state of the game controller.
    /// This raises an event when a subscription is initially established.
    /// </summary>
    [Description("Changes (or indcates a change to) the state of the controller.\n(Also generated when this service is started.)")]
    public class Replace : Replace<XInputGamepadState, PortSet<DefaultReplaceResponseType, Fault>>
    {
    }

    [Description("Polls the controller to update the current state.\n If the state has changed, then appropriate notifications are sent.")]
    public class Poll : Submit<PollRequest, PortSet<DefaultSubmitResponseType, Fault>>
    {
        public Poll()
        {
        }

        public Poll(PollRequest body)
            : base(body)
        {

        }
    }
    /// <summary>
    /// Send this message to change which controller this service instance is using.
    /// This event is raised when the controller is changed or when the controller is connected or disconnected
    /// </summary>
    [Description("Sets the controller (or indicates when the controller has changed or has been connected or disconnected).")]
    public class ControllerChanged : Update<Controller, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public ControllerChanged()
        {
        }

        public ControllerChanged(Controller body)
            : base(body)
        {
        }
    }

    [Description("Indicates when the directional pad position changes.")]
    public class DPadChanged : Update<DPad, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public DPadChanged()
        {
        }

        public DPadChanged(DPad body)
            : base(body)
        {
        }
    }

    [Description("Indicates when one or more buttons are pressed or released on the controller.")]
    public class ButtonsChanged : Update<Buttons, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public ButtonsChanged()
        {
        }

        public ButtonsChanged(Buttons body)
            : base(body)
        {
        }
    }

    [Description("Indicates when a trigger's value when it is pressed.")]
    public class TriggersChanged : Update<Triggers, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public TriggersChanged()
        {
        }

        public TriggersChanged(Triggers body)
            : base(body)
        {
        }
    }

    [DisplayName("ThumbsticksChange")]
    [Description("Indicates when a thumbstick position changes.")]
    public class ThumbsticksChanged : Update<Thumbsticks, PortSet<DefaultUpdateResponseType, Fault>>
    {
        public ThumbsticksChanged()
        {
        }

        public ThumbsticksChanged(Thumbsticks body)
            : base(body)
        {
        }
    }
    /// <summary>
    /// Sets the amount of vibration in the left and right rumble motors. 
    /// The left motor is the low-frequency rumble motor. 
    /// The right motor is the high-frequency rumble motor.
    /// This notification is raised when the vibration settings are changed.
    /// </summary>
    [DisplayName("SetVibrationMotor")]
    [Description("Sets or indicates when the controller vibration motor settings are changed.\nThe left motor provides a low-frequency vibration; the right motor, high frequency.")]
    public class SetVibration : Update<Vibration, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    [DisplayName("QueryControllers")]
    [Description("Gets the list of attached controllers.")]
    public class QueryControllers : Query<QueryControllersRequest, PortSet<QueryControllersResponse, Fault>>
    {
    }
}
